@@ -30,12 +30,11 @@
 
 #include "seahorse-certificate.h"
 #include "seahorse-pkcs11.h"
-#include "seahorse-pkcs11-actions.h"
 #include "seahorse-pkcs11-helpers.h"
-#include "seahorse-pkcs11-operations.h"
 #include "seahorse-private-key.h"
 #include "seahorse-token.h"
 
+#include "seahorse-lockable.h"
 #include "seahorse-place.h"
 #include "seahorse-registry.h"
 #include "seahorse-util.h"
@@ -59,8 +58,8 @@ struct _SeahorseTokenPrivate {
 	GckSlot *slot;
 	gchar *uri;
 	GckTokenInfo *info;
+	GArray *mechanisms;
 	GckSession *session;
-	GtkActionGroup *actions;
 	GHashTable *object_for_handle;
 	GHashTable *objects_for_id;
 	GHashTable *id_for_object;
@@ -71,11 +70,25 @@ static void          seahorse_token_place_iface          (SeahorsePlaceIface *if
 
 static void          seahorse_token_collection_iface     (GcrCollectionIface *iface);
 
+static void          seahorse_token_lockable_iface       (SeahorseLockableIface *iface);
+
 G_DEFINE_TYPE_EXTENDED (SeahorseToken, seahorse_token, G_TYPE_OBJECT, 0,
                         G_IMPLEMENT_INTERFACE (GCR_TYPE_COLLECTION, seahorse_token_collection_iface);
                         G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_PLACE, seahorse_token_place_iface);
+                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_LOCKABLE, seahorse_token_lockable_iface);
 );
 
+static void
+update_mechanisms (SeahorseToken *self)
+{
+	GArray *mechanisms;
+
+	mechanisms = gck_slot_get_mechanisms (self->pv->slot);
+	if (self->pv->mechanisms != NULL)
+		g_array_unref (self->pv->mechanisms);
+	self->pv->mechanisms = mechanisms;
+}
+
 static void
 update_token_info (SeahorseToken *self)
 {
@@ -97,7 +110,7 @@ update_token_info (SeahorseToken *self)
 static void
 update_id_map (SeahorseToken *self,
                gpointer object,
-               GckAttribute *id)
+               const GckAttribute *id)
 {
 	GPtrArray *objects;
 	GckAttribute *pid;
@@ -126,11 +139,11 @@ update_id_map (SeahorseToken *self,
 		if (!g_hash_table_lookup_extended (self->pv->objects_for_id, id,
 		                                   (gpointer *)&id, (gpointer *)&objects)) {
 			objects = g_ptr_array_new ();
-			id = gck_attribute_dup (id);
-			g_hash_table_insert (self->pv->objects_for_id, id, objects);
+			g_hash_table_insert (self->pv->objects_for_id,
+			                     gck_attribute_dup (id), objects);
 		}
 		g_ptr_array_add (objects, object);
-		g_hash_table_insert (self->pv->id_for_object, object, id);
+		g_hash_table_insert (self->pv->id_for_object, object, (GckAttribute *)id);
 	}
 
 	/* Remove this object from the map */
@@ -153,11 +166,14 @@ update_id_map (SeahorseToken *self,
 static gpointer
 lookup_id_map (SeahorseToken *self,
                GType object_type,
-               GckAttribute *id)
+               const GckAttribute *id)
 {
 	GPtrArray *objects;
 	guint i;
 
+	if (id == NULL)
+		return NULL;
+
 	objects = g_hash_table_lookup (self->pv->objects_for_id, id);
 	if (objects == NULL)
 		return NULL;
@@ -233,7 +249,7 @@ receive_objects (SeahorseToken *self,
                  GList *objects)
 {
 	GckAttributes *attrs;
-	GckAttribute *id;
+	const GckAttribute *id;
 	gpointer object;
 	gpointer prev;
 	gpointer pair;
@@ -245,7 +261,7 @@ receive_objects (SeahorseToken *self,
 	for (l = objects; l != NULL; l = g_list_next (l)) {
 		object = l->data;
 		handle = gck_object_get_handle (object);
-		attrs = gck_object_attributes_get_attributes (object);
+		attrs = gck_object_cache_get_attributes (object);
 
 		prev = g_hash_table_lookup (self->pv->object_for_handle, &handle);
 		if (prev == NULL) {
@@ -254,11 +270,11 @@ receive_objects (SeahorseToken *self,
 			                     g_object_ref (object));
 			g_object_set (object, "place", self, NULL);
 		} else if (prev != object) {
-			gck_object_attributes_set_attributes (prev, attrs);
+			gck_object_cache_set_attributes (prev, attrs);
 			object = prev;
 		}
 
-		id = gck_attributes_find (attrs, CKA_ID);
+		id = attrs ? gck_attributes_find (attrs, CKA_ID) : NULL;
 		update_id_map (self, object, id);
 
 		if (SEAHORSE_IS_CERTIFICATE (object)) {
@@ -392,28 +408,24 @@ static void
 refresh_enumerate (GSimpleAsyncResult *res)
 {
 	RefreshClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GckBuilder builder = GCK_BUILDER_INIT;
 	GckSession *session;
 	GckEnumerator *enumerator;
-	GckAttributes *attrs;
 
 	session = seahorse_token_get_session (closure->token);
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_CERTIFICATE);
-	enumerator = gck_session_enumerate_objects (session, attrs);
+	gck_builder_add_boolean (&builder, CKA_TOKEN, TRUE);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_CERTIFICATE);
+	enumerator = gck_session_enumerate_objects (session, gck_builder_end (&builder));
 	gck_enumerator_set_object_type (enumerator, SEAHORSE_TYPE_CERTIFICATE);
 	closure->enumerator = enumerator;
-	gck_attributes_unref (attrs);
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_PRIVATE_KEY);
-	enumerator = gck_session_enumerate_objects (session, attrs);
+	gck_builder_add_boolean (&builder, CKA_TOKEN, TRUE);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_PRIVATE_KEY);
+	enumerator = gck_session_enumerate_objects (session, gck_builder_end (&builder));
 	gck_enumerator_set_object_type (enumerator, SEAHORSE_TYPE_PRIVATE_KEY);
 	gck_enumerator_set_chained (closure->enumerator, enumerator);
 	g_object_unref (enumerator);
-	gck_attributes_unref (attrs);
 
 	gck_enumerator_next_async (closure->enumerator, 16, closure->cancellable,
 	                           on_refresh_next_objects, g_object_ref (res));
@@ -453,24 +465,23 @@ calculate_session_options (SeahorseToken *self)
 		return GCK_SESSION_READ_WRITE;
 }
 
-void
-seahorse_token_refresh_async (SeahorseToken *self,
-                              GCancellable *cancellable,
-                              GAsyncReadyCallback callback,
-                              gpointer user_data)
+static void
+seahorse_token_load_async (SeahorsePlace *place,
+                           GCancellable *cancellable,
+                           GAsyncReadyCallback callback,
+                           gpointer user_data)
 {
+	SeahorseToken *self = SEAHORSE_TOKEN (place);
 	GSimpleAsyncResult *res;
 	RefreshClosure *closure;
 	GckSessionOptions options;
 	GList *objects, *l;
 	gulong handle;
 
-	g_return_if_fail (SEAHORSE_IS_TOKEN (self));
-
 	update_token_info (self);
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_token_refresh_async);
+	                                 seahorse_token_load_async);
 	closure = g_new0 (RefreshClosure, 1);
 	closure->checks = g_hash_table_new_full (seahorse_pkcs11_ulong_hash,
 	                                         seahorse_pkcs11_ulong_equal,
@@ -500,13 +511,13 @@ seahorse_token_refresh_async (SeahorseToken *self,
 	g_object_unref (res);
 }
 
-gboolean
-seahorse_token_refresh_finish (SeahorseToken *token,
-                               GAsyncResult *result,
-                               GError **error)
+static gboolean
+seahorse_token_load_finish (SeahorsePlace *place,
+                            GAsyncResult *result,
+                            GError **error)
 {
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (token),
-	                      seahorse_token_refresh_async), FALSE);
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (place),
+	                      seahorse_token_load_async), FALSE);
 
 	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
 		return FALSE;
@@ -519,7 +530,6 @@ static void
 seahorse_token_init (SeahorseToken *self)
 {
 	self->pv = (G_TYPE_INSTANCE_GET_PRIVATE (self, SEAHORSE_TYPE_TOKEN, SeahorseTokenPrivate));
-	self->pv->actions = seahorse_pkcs11_token_actions_instance ();
 	self->pv->object_for_handle = g_hash_table_new_full (seahorse_pkcs11_ulong_hash,
 	                                                     seahorse_pkcs11_ulong_equal,
 	                                                     g_free, g_object_unref);
@@ -541,7 +551,7 @@ seahorse_token_constructed (GObject *obj)
 
 	g_return_if_fail (self->pv->slot != NULL);
 
-	seahorse_token_refresh_async (self, NULL, NULL, NULL);
+	seahorse_place_load_async (SEAHORSE_PLACE (self), NULL, NULL, NULL);
 
 	data = gck_uri_data_new ();
 	data->token_info = seahorse_token_get_info (self);
@@ -594,7 +604,7 @@ seahorse_token_get_property (GObject *object,
 		g_value_set_string (value, self->pv->uri);
 		break;
 	case PROP_ACTIONS:
-		g_value_set_object (value, self->pv->actions);
+		g_value_set_object (value, NULL);
 		break;
 	case PROP_INFO:
 		g_value_set_boxed (value, self->pv->info);
@@ -663,7 +673,6 @@ seahorse_token_finalize (GObject *obj)
 	g_hash_table_destroy (self->pv->objects_for_id);
 	g_hash_table_destroy (self->pv->id_for_object);
 	g_assert (self->pv->slot == NULL);
-	g_clear_object (&self->pv->actions);
 	g_free (self->pv->uri);
 
 	G_OBJECT_CLASS (seahorse_token_parent_class)->finalize (obj);
@@ -716,7 +725,8 @@ seahorse_token_class_init (SeahorseTokenClass *klass)
 static void
 seahorse_token_place_iface (SeahorsePlaceIface *iface)
 {
-
+	iface->load_async = seahorse_token_load_async;
+	iface->load_finish = seahorse_token_load_finish;
 }
 
 static guint
@@ -749,9 +759,177 @@ seahorse_token_collection_iface (GcrCollectionIface *iface)
 	iface->contains = seahorse_token_contains;
 }
 
-/* --------------------------------------------------------------------------
- * PUBLIC
- */
+static gboolean
+is_session_logged_in (GckSession *session)
+{
+	GckSessionInfo *info;
+	gboolean logged_in;
+
+	if (session == NULL)
+		return FALSE;
+
+	info = gck_session_get_info (session);
+	logged_in = (info != NULL) &&
+	            (info->state == CKS_RW_USER_FUNCTIONS ||
+	             info->state == CKS_RO_USER_FUNCTIONS ||
+	             info->state == CKS_RW_SO_FUNCTIONS);
+	gck_session_info_free (info);
+
+	return logged_in;
+}
+
+static void
+on_session_logout (GObject *source,
+                   GAsyncResult *result,
+                   gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
+	GError *error = NULL;
+
+	gck_session_logout_finish (GCK_SESSION (source), result, &error);
+	if (error == NULL)
+		seahorse_place_load_async (SEAHORSE_PLACE (self), NULL, NULL, NULL);
+	else
+		g_simple_async_result_take_error (res, error);
+
+	g_simple_async_result_complete (res);
+	g_object_unref (self);
+	g_object_unref (res);
+}
+
+static void
+on_login_interactive (GObject *source,
+                      GAsyncResult *result,
+                      gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
+	GError *error = NULL;
+
+	gck_session_logout_finish (GCK_SESSION (source), result, &error);
+	if (error == NULL)
+		seahorse_token_load_async (SEAHORSE_PLACE (self), NULL, NULL, NULL);
+	else
+		g_simple_async_result_take_error (res, error);
+
+	g_simple_async_result_complete (res);
+	g_object_unref (self);
+	g_object_unref (res);
+}
+
+static void
+on_session_login_open (GObject *source,
+                       GAsyncResult *result,
+                       gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
+	GckSession *session;
+	GError *error = NULL;
+
+	session = gck_session_open_finish (result, &error);
+	if (error == NULL) {
+		seahorse_token_set_session (self, session);
+		seahorse_place_load_async (SEAHORSE_PLACE (self), NULL, NULL, NULL);
+		g_object_unref (session);
+	} else {
+		g_simple_async_result_take_error (res, error);
+	}
+
+	g_simple_async_result_complete (res);
+	g_object_unref (self);
+	g_object_unref (res);
+}
+
+static void
+seahorse_token_lock_async (SeahorseLockable *lockable,
+                           GTlsInteraction *interaction,
+                           GCancellable *cancellable,
+                           GAsyncReadyCallback callback,
+                           gpointer user_data)
+{
+	SeahorseToken *self = SEAHORSE_TOKEN (lockable);
+	GSimpleAsyncResult *res;
+
+	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
+	                                 seahorse_token_lock_async);
+
+	if (is_session_logged_in (self->pv->session))
+		gck_session_logout_async (self->pv->session, cancellable,
+		                          on_session_logout, g_object_ref (res));
+	else
+		g_simple_async_result_complete_in_idle (res);
+
+	g_object_unref (res);
+}
+
+static gboolean
+seahorse_token_lock_finish (SeahorseLockable *lockable,
+                            GAsyncResult *result,
+                            GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (lockable),
+	                      seahorse_token_lock_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+seahorse_token_unlock_async (SeahorseLockable *lockable,
+                             GTlsInteraction *interaction,
+                             GCancellable *cancellable,
+                             GAsyncReadyCallback callback,
+                             gpointer user_data)
+{
+	SeahorseToken *self = SEAHORSE_TOKEN (lockable);
+	GSimpleAsyncResult *res;
+	GckSessionOptions options;
+
+	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
+	                                 seahorse_token_unlock_async);
+
+	if (is_session_logged_in (self->pv->session)) {
+		g_simple_async_result_complete_in_idle (res);
+
+	} else if (self->pv->session) {
+		gck_session_login_interactive_async (self->pv->session, CKU_USER,
+		                                     interaction, NULL, on_login_interactive,
+		                                     g_object_ref (res));
+	} else {
+		options = calculate_session_options (self);
+		gck_session_open_async (self->pv->slot, options | GCK_SESSION_LOGIN_USER, interaction,
+		                        NULL, on_session_login_open, g_object_ref (res));
+	}
+
+	g_object_unref (res);
+}
+
+static gboolean
+seahorse_token_unlock_finish (SeahorseLockable *lockable,
+                              GAsyncResult *result,
+                              GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (lockable),
+	                      seahorse_token_unlock_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+seahorse_token_lockable_iface (SeahorseLockableIface *iface)
+{
+	iface->lock_async = seahorse_token_lock_async;
+	iface->lock_finish = seahorse_token_lock_finish;
+	iface->unlock_async = seahorse_token_unlock_async;
+	iface->unlock_finish = seahorse_token_unlock_finish;
+}
 
 SeahorseToken *
 seahorse_token_new (GckSlot *slot)
@@ -808,25 +986,6 @@ seahorse_token_get_info (SeahorseToken *self)
 	return self->pv->info;
 }
 
-static gboolean
-is_session_logged_in (GckSession *session)
-{
-	GckSessionInfo *info;
-	gboolean logged_in;
-
-	if (session == NULL)
-		return FALSE;
-
-	info = gck_session_get_info (session);
-	logged_in = (info != NULL) &&
-	            (info->state == CKS_RW_USER_FUNCTIONS ||
-	             info->state == CKS_RO_USER_FUNCTIONS ||
-	             info->state == CKS_RW_SO_FUNCTIONS);
-	gck_session_info_free (info);
-
-	return logged_in;
-}
-
 gboolean
 seahorse_token_get_lockable (SeahorseToken *self)
 {
@@ -864,6 +1023,31 @@ seahorse_token_get_unlockable (SeahorseToken *self)
 	return !is_session_logged_in (self->pv->session);
 }
 
+GArray *
+seahorse_token_get_mechanisms (SeahorseToken *self)
+{
+	g_return_val_if_fail (SEAHORSE_IS_TOKEN (self), NULL);
+
+	if (!self->pv->mechanisms)
+		update_mechanisms (self);
+
+	return self->pv->mechanisms;
+}
+
+gboolean
+seahorse_token_has_mechanism (SeahorseToken *self,
+                              gulong mechanism)
+{
+	g_return_val_if_fail (SEAHORSE_IS_TOKEN (self), FALSE);
+
+	if (!self->pv->mechanisms)
+		update_mechanisms (self);
+
+	return gck_mechanisms_check (self->pv->mechanisms,
+	                             mechanism,
+	                             GCK_INVALID);
+}
+
 void
 seahorse_token_remove_object (SeahorseToken *self,
                               GckObject *object)
@@ -878,156 +1062,26 @@ seahorse_token_remove_object (SeahorseToken *self,
 	g_list_free_full (objects, g_object_unref);
 }
 
-static void
-on_session_logout (GObject *source,
-                   GAsyncResult *result,
-                   gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
-	GError *error = NULL;
-
-	gck_session_logout_finish (GCK_SESSION (source), result, &error);
-	if (error == NULL)
-		seahorse_token_refresh_async (self, NULL, NULL, NULL);
-	else
-		g_simple_async_result_take_error (res, error);
-
-	g_simple_async_result_complete (res);
-	g_object_unref (self);
-	g_object_unref (res);
-}
-
-void
-seahorse_token_lock_async (SeahorseToken *self,
-                           GTlsInteraction *interaction,
-                           GCancellable *cancellable,
-                           GAsyncReadyCallback callback,
-                           gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-
-	g_return_if_fail (SEAHORSE_IS_TOKEN (self));
-	g_return_if_fail (interaction == NULL || G_IS_TLS_INTERACTION (interaction));
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_token_lock_async);
-
-	if (is_session_logged_in (self->pv->session))
-		gck_session_logout_async (self->pv->session, cancellable,
-		                          on_session_logout, g_object_ref (res));
-	else
-		g_simple_async_result_complete_in_idle (res);
-
-	g_object_unref (res);
-}
-
 gboolean
-seahorse_token_lock_finish (SeahorseToken *self,
-                            GAsyncResult *result,
-                            GError **error)
+seahorse_token_is_deletable (SeahorseToken *self,
+                             GckObject *object)
 {
-	g_return_val_if_fail (SEAHORSE_IS_TOKEN (self), FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
-	                      seahorse_token_lock_async), FALSE);
+	GckAttributes *attrs;
+	GckTokenInfo *info;
+	gboolean ret;
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+	g_return_val_if_fail (SEAHORSE_IS_TOKEN (self), FALSE);
+	g_return_val_if_fail (GCK_IS_OBJECT (object), FALSE);
+
+	info = seahorse_token_get_info (self);
+	if (info->flags & CKF_WRITE_PROTECTED)
 		return FALSE;
 
-	return TRUE;
-}
+	g_object_get (object, "attributes", &attrs, NULL);
 
-static void
-on_login_interactive (GObject *source,
-                      GAsyncResult *result,
-                      gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
-	GError *error = NULL;
+	if (!gck_attributes_find_boolean (attrs, CKA_MODIFIABLE, &ret))
+		ret = TRUE;
 
-	gck_session_logout_finish (GCK_SESSION (source), result, &error);
-	if (error == NULL)
-		seahorse_token_refresh_async (self, NULL, NULL, NULL);
-	else
-		g_simple_async_result_take_error (res, error);
-
-	g_simple_async_result_complete (res);
-	g_object_unref (self);
-	g_object_unref (res);
-}
-
-static void
-on_session_login_open (GObject *source,
-                       GAsyncResult *result,
-                       gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	SeahorseToken *self = SEAHORSE_TOKEN (g_async_result_get_source_object (user_data));
-	GckSession *session;
-	GError *error = NULL;
-
-	session = gck_session_open_finish (result, &error);
-	if (error == NULL) {
-		seahorse_token_set_session (self, session);
-		seahorse_token_refresh_async (self, NULL, NULL, NULL);
-		g_object_unref (session);
-	} else {
-		g_simple_async_result_take_error (res, error);
-	}
-
-	g_simple_async_result_complete (res);
-	g_object_unref (self);
-	g_object_unref (res);
-}
-
-void
-seahorse_token_unlock_async (SeahorseToken *self,
-                             GTlsInteraction *interaction,
-                             GCancellable *cancellable,
-                             GAsyncReadyCallback callback,
-                             gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-	GckSessionOptions options;
-
-	g_return_if_fail (SEAHORSE_IS_TOKEN (self));
-	g_return_if_fail (interaction == NULL || G_IS_TLS_INTERACTION (interaction));
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_token_unlock_async);
-
-	if (is_session_logged_in (self->pv->session)) {
-		g_simple_async_result_complete_in_idle (res);
-
-	} else if (self->pv->session) {
-		gck_session_login_interactive_async (self->pv->session, CKU_USER,
-		                                     interaction, NULL, on_login_interactive,
-		                                     g_object_ref (res));
-	} else {
-		options = calculate_session_options (self);
-		gck_session_open_async (self->pv->slot, options | GCK_SESSION_LOGIN_USER, interaction,
-		                        NULL, on_session_login_open, g_object_ref (res));
-	}
-
-	g_object_unref (res);
-}
-
-gboolean
-seahorse_token_unlock_finish (SeahorseToken *self,
-                              GAsyncResult *result,
-                              GError **error)
-{
-	g_return_val_if_fail (SEAHORSE_IS_TOKEN (self), FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
-	                      seahorse_token_unlock_async), FALSE);
-
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
+	gck_attributes_unref (attrs);
+	return ret;
 }
